@@ -16,13 +16,14 @@ import os
 
 from eventlet.green import socket
 from oslo_config import cfg
+from oslo_db import options as oslo_db_ops
 from oslo_log import log as logging
 from oslo_middleware import cors
+from oslo_policy import opts as policy_opts
 from osprofiler import opts as profiler
 
 from heat.common import exception
 from heat.common.i18n import _
-from heat.common.i18n import _LW
 from heat.common import wsgi
 
 
@@ -46,9 +47,6 @@ service_opts = [
                       'keystone catalog')),
     cfg.StrOpt('heat_waitcondition_server_url',
                help=_('URL of the Heat waitcondition server.')),
-    cfg.StrOpt('heat_watch_server_url',
-               default="",
-               help=_('URL of the Heat CloudWatch server.')),
     cfg.StrOpt('instance_connection_is_secure',
                default="0",
                help=_('Instance connection to CFN/CW API via https.')),
@@ -58,6 +56,13 @@ service_opts = [
                       'SSL is used.')),
     cfg.StrOpt('region_name_for_services',
                help=_('Default region name used to get services endpoints.')),
+    cfg.StrOpt('region_name_for_shared_services',
+               help=_('Region name for shared services endpoints.')),
+    cfg.ListOpt('shared_services_types',
+                default=['image', 'volume', 'volumev3'],
+                help=_('The shared services located in the other region.'
+                       'Needs region_name_for_shared_services option to '
+                       'be set for this to take effect.')),
     cfg.StrOpt('heat_stack_user_role',
                default="heat_stack_user",
                help=_('Keystone role for heat template-defined users.')),
@@ -86,7 +91,14 @@ service_opts = [
     cfg.IntOpt('num_engine_workers',
                help=_('Number of heat-engine processes to fork and run. '
                       'Will default to either to 4 or number of CPUs on '
-                      'the host, whichever is greater.'))]
+                      'the host, whichever is greater.')),
+    cfg.StrOpt('server_keystone_endpoint_type',
+               choices=['', 'public', 'internal', 'admin'],
+               default='',
+               help=_('If set, is used to control which authentication '
+                      'endpoint is used by user-controlled servers to make '
+                      'calls back to Heat. '
+                      'If unset www_authenticate_uri is used.'))]
 
 engine_opts = [
     cfg.ListOpt('plugin_dirs',
@@ -102,6 +114,11 @@ engine_opts = [
     cfg.StrOpt('deferred_auth_method',
                choices=['password', 'trusts'],
                default='trusts',
+               deprecated_for_removal=True,
+               deprecated_reason='Stored password based deferred auth is '
+                                 'broken when used with keystone v3 and '
+                                 'is not supported.',
+               deprecated_since='9.0.0',
                help=_('Select deferred auth method, '
                       'stored password or trusts.')),
     cfg.StrOpt('reauthentication_auth_method',
@@ -110,10 +127,18 @@ engine_opts = [
                help=_('Allow reauthentication on token expiry, such that'
                       ' long-running tasks may complete. Note this defeats'
                       ' the expiry of any provided user tokens.')),
-    cfg.IntOpt('stale_token_duration',
-               default=30,
-               help=_('Gap, in seconds, to determine whether the given token '
-                      'is about to expire.'),),
+    cfg.BoolOpt('allow_trusts_redelegation',
+                default=False,
+                help=_('Create trusts with redelegation enabled. '
+                       'This option is only used when '
+                       'reauthentication_auth_method is set to "trusts". '
+                       'Note that enabling this option does have '
+                       'security implications as all trusts created by Heat '
+                       'will use both impersonation and redelegation enabled. '
+                       'Enable it only when there are other services that '
+                       'need to create trusts from tokens Heat uses to '
+                       'access them, examples are Aodh and Heat in another '
+                       'region when configured to use trusts too.')),
     cfg.ListOpt('trusts_delegated_roles',
                 default=[],
                 help=_('Subset of trustor roles to be delegated to heat.'
@@ -124,9 +149,9 @@ engine_opts = [
                help=_('Maximum resources allowed per top-level stack. '
                       '-1 stands for unlimited.')),
     cfg.IntOpt('max_stacks_per_tenant',
-               default=100,
-               help=_('Maximum number of stacks any one tenant may have'
-                      ' active at one time.')),
+               default=512,
+               help=_('Maximum number of stacks any one tenant may have '
+                      'active at one time. -1 stands for unlimited.')),
     cfg.IntOpt('action_retry_limit',
                default=5,
                help=_('Number of times to retry to bring a '
@@ -149,17 +174,31 @@ engine_opts = [
                default=10,
                help=_('Number of times to check whether an interface has '
                       'been attached or detached.')),
+    cfg.FloatOpt('max_nova_api_microversion',
+                 help=_('Maximum nova API version for client plugin. With '
+                        'this limitation, any nova feature supported with '
+                        'microversion number above max_nova_api_microversion '
+                        'will not be available.')),
+    cfg.FloatOpt('max_ironic_api_microversion',
+                 help=_('Maximum ironic API version for client plugin. With '
+                        'this limitation, any ironic feature supported with '
+                        'microversion number above '
+                        'max_ironic_api_microversion will not be available.')),
     cfg.IntOpt('event_purge_batch_size',
-               default=10,
+               min=1,
+               default=200,
                help=_("Controls how many events will be pruned whenever a "
-                      "stack's events exceed max_events_per_stack. Set this "
+                      "stack's events are purged. Set this "
                       "lower to keep more events at the expense of more "
                       "frequent purges.")),
     cfg.IntOpt('max_events_per_stack',
                default=1000,
-               help=_('Maximum events that will be available per stack. Older'
-                      ' events will be deleted when this is reached. Set to 0'
-                      ' for unlimited events per stack.')),
+               help=_('Rough number of maximum events that will be available '
+                      'per stack. Actual number of events can be a bit '
+                      'higher since purge checks take place randomly '
+                      '200/event_purge_batch_size percent of the time. '
+                      'Older events are deleted when events are purged. '
+                      'Set to 0 for unlimited events per stack.')),
     cfg.IntOpt('stack_action_timeout',
                default=3600,
                help=_('Timeout in seconds for stack action (ie. create or'
@@ -173,9 +212,6 @@ engine_opts = [
                default=2,
                help=_('RPC timeout for the engine liveness check that is used'
                       ' for stack locking.')),
-    cfg.BoolOpt('enable_cloud_watch_lite',
-                default=False,
-                help=_('Enable the legacy OS::Heat::CWLiteAlarm resource.')),
     cfg.BoolOpt('enable_stack_abandon',
                 default=False,
                 help=_('Enable the preview Stack Abandon feature.')),
@@ -230,6 +266,19 @@ engine_opts = [
                       'credentials. ZAQAR_SIGNAL will create a dedicated '
                       'zaqar queue to be signaled using the provided keystone '
                       'credentials.')),
+    cfg.StrOpt('default_user_data_format',
+               choices=['HEAT_CFNTOOLS',
+                        'RAW',
+                        'SOFTWARE_CONFIG'],
+               default='HEAT_CFNTOOLS',
+               help=_('Template default for how the user_data should be '
+                      'formatted for the server. For HEAT_CFNTOOLS, the '
+                      'user_data is bundled as part of the heat-cfntools '
+                      'cloud-init boot configuration data. For RAW the '
+                      'user_data is passed to Nova unmodified. For '
+                      'SOFTWARE_CONFIG user_data is bundled as part of the '
+                      'software config data, and metadata is derived from any '
+                      'associated SoftwareDeployment resources.')),
     cfg.ListOpt('hidden_stack_tags',
                 default=['data-processing-cluster'],
                 help=_('Stacks containing these tag names will be hidden. '
@@ -264,6 +313,7 @@ engine_opts = [
 rpc_opts = [
     cfg.StrOpt('host',
                default=socket.gethostname(),
+               sample_default='<Hostname>',
                help=_('Name of the engine node. '
                       'This can be an opaque identifier. '
                       'It is not necessarily a hostname, FQDN, '
@@ -352,14 +402,21 @@ volumes_opts = [
                        "This is a temporary workaround until cinder-backup "
                        "service becomes discoverable, see LP#1334856."))]
 
+noauth_group = cfg.OptGroup('noauth')
+noauth_opts = [
+    cfg.StrOpt('token_response',
+               default='',
+               help=_("JSON file containing the content returned by the "
+                      "noauth middleware."))]
+
 
 def startup_sanity_check():
     if (not cfg.CONF.stack_user_domain_id and
             not cfg.CONF.stack_user_domain_name):
         # FIXME(shardy): Legacy fallback for folks using old heat.conf
         # files which lack domain configuration
-        LOG.warning(_LW('stack_user_domain_id or stack_user_domain_name not '
-                        'set in heat.conf falling back to using default'))
+        LOG.warning('stack_user_domain_id or stack_user_domain_name not '
+                    'set in heat.conf falling back to using default')
     else:
         domain_admin_user = cfg.CONF.stack_domain_admin
         domain_admin_password = cfg.CONF.stack_domain_admin_password
@@ -372,7 +429,7 @@ def startup_sanity_check():
     auth_key_len = len(cfg.CONF.auth_encryption_key)
     if auth_key_len in (16, 24):
         LOG.warning(
-            _LW('Please update auth_encryption_key to be 32 characters.'))
+            'Please update auth_encryption_key to be 32 characters.')
     elif auth_key_len != 32:
         raise exception.Error(_('heat.conf misconfigured, auth_encryption_key '
                                 'must be 32 characters'))
@@ -386,13 +443,14 @@ def list_opts():
     yield auth_password_group.name, auth_password_opts
     yield revision_group.name, revision_opts
     yield volumes_group.name, volumes_opts
+    yield noauth_group.name, noauth_opts
     yield profiler.list_opts()[0]
     yield 'clients', default_clients_opts
 
-    for client in ('aodh', 'barbican', 'ceilometer', 'cinder', 'designate',
+    for client in ('aodh', 'barbican', 'cinder', 'designate',
                    'glance', 'heat', 'keystone', 'magnum', 'manila', 'mistral',
-                   'monasca', 'neutron', 'nova', 'sahara', 'senlin', 'swift',
-                   'trove', 'zaqar'
+                   'monasca', 'neutron', 'nova', 'octavia', 'sahara', 'senlin',
+                   'swift', 'trove', 'vitrage', 'zaqar'
                    ):
         client_specific_group = 'clients_' + client
         yield client_specific_group, clients_opts
@@ -401,6 +459,7 @@ def list_opts():
     yield 'clients_keystone', keystone_client_opts
     yield 'clients_nova', client_http_log_debug_opts
     yield 'clients_cinder', client_http_log_debug_opts
+    yield oslo_db_ops.list_opts()[0]
 
 
 cfg.CONF.register_group(paste_deploy_group)
@@ -442,7 +501,7 @@ def load_paste_app(app_name=None):
 
     :param app_name: name of the application to load
 
-    :raises RuntimeError when config file cannot be located or application
+    :raises RuntimeError: when config file cannot be located or application
             cannot be loaded from config file
     """
     if app_name is None:
@@ -508,23 +567,25 @@ def get_ssl_options(client):
 
 def set_config_defaults():
     """This method updates all configuration default values."""
-    # CORS Defaults
-    # TODO(krotscheck): Update with https://review.openstack.org/#/c/285368/
-    cfg.set_defaults(cors.CORS_OPTS,
-                     allow_headers=['X-Auth-Token',
-                                    'X-Identity-Status',
-                                    'X-Roles',
-                                    'X-Service-Catalog',
-                                    'X-User-Id',
-                                    'X-Tenant-Id',
-                                    'X-OpenStack-Request-ID'],
-                     expose_headers=['X-Auth-Token',
-                                     'X-Subject-Token',
-                                     'X-Service-Token',
-                                     'X-OpenStack-Request-ID'],
-                     allow_methods=['GET',
-                                    'PUT',
-                                    'POST',
-                                    'DELETE',
-                                    'PATCH']
-                     )
+    cors.set_defaults(
+        allow_headers=['X-Auth-Token',
+                       'X-Identity-Status',
+                       'X-Roles',
+                       'X-Service-Catalog',
+                       'X-User-Id',
+                       'X-Tenant-Id',
+                       'X-OpenStack-Request-ID'],
+        expose_headers=['X-Auth-Token',
+                        'X-Subject-Token',
+                        'X-Service-Token',
+                        'X-OpenStack-Request-ID'],
+        allow_methods=['GET',
+                       'PUT',
+                       'POST',
+                       'DELETE',
+                       'PATCH']
+    )
+    # TODO(gmann): Remove setting the default value of config policy_file
+    # once oslo_policy change the default value to 'policy.yaml'.
+    # https://github.com/openstack/oslo.policy/blob/a626ad12fe5a3abd49d70e3e5b95589d279ab578/oslo_policy/opts.py#L49
+    policy_opts.set_defaults(cfg.CONF, 'policy.yaml')
