@@ -49,6 +49,12 @@ CONF = cfg.CONF
 CONF.import_opt('hidden_stack_tags', 'heat.common.config')
 CONF.import_opt('max_events_per_stack', 'heat.common.config')
 CONF.import_group('profiler', 'heat.common.config')
+CONF.import_opt('db_max_retries', 'oslo_db.options', group='database')
+CONF.import_opt('db_retry_interval', 'oslo_db.options', group='database')
+CONF.import_opt(
+    'db_inc_retry_interval', 'oslo_db.options', group='database')
+CONF.import_opt(
+    'db_max_retry_interval', 'oslo_db.options', group='database')
 
 options.set_defaults(CONF)
 
@@ -59,7 +65,9 @@ LOG = logging.getLogger(__name__)
 
 
 # TODO(sbaker): fix tests so that sqlite_fk=True can be passed to configure
-db_context.configure()
+# FIXME(stephenfin): we need to remove reliance on autocommit semantics ASAP
+# since it's not compatible with SQLAlchemy 2.0
+db_context.configure(__autocommit=True)
 
 
 def get_facade():
@@ -94,11 +102,13 @@ def retry_on_db_error(func):
     def try_func(context, *args, **kwargs):
         if (context.session.transaction is None or
                 not context.session.autocommit):
-            wrapped = oslo_db_api.wrap_db_retry(max_retries=3,
-                                                retry_on_deadlock=True,
-                                                retry_on_disconnect=True,
-                                                retry_interval=0.5,
-                                                inc_retry_interval=True)(func)
+            wrapped = oslo_db_api.wrap_db_retry(
+                max_retries=CONF.database.db_max_retries,
+                retry_on_deadlock=True,
+                retry_on_disconnect=True,
+                retry_interval=CONF.database.db_retry_interval,
+                inc_retry_interval=CONF.database.db_inc_retry_interval,
+                max_retry_interval=CONF.database.db_max_retry_interval)(func)
             return wrapped(context, *args, **kwargs)
         else:
             try:
@@ -486,7 +496,7 @@ def resource_create(context, values):
 
 @retry_on_db_error
 def resource_create_replacement(context,
-                                existing_res_id, existing_res_values,
+                                existing_res_id,
                                 new_res_values,
                                 atomic_key, expected_engine_id=None):
     session = context.session
@@ -494,7 +504,6 @@ def resource_create_replacement(context,
         with session.begin():
             new_res = resource_create(context, new_res_values)
             update_data = {'replaced_by': new_res.id}
-            update_data.update(existing_res_values)
             if not _try_resource_update(context,
                                         existing_res_id, update_data,
                                         atomic_key,
@@ -1132,7 +1141,7 @@ def _delete_event_rows(context, stack_id, limit):
         (ids, rsrc_prop_ids) = zip(*id_pairs)
         max_id = ids[-1]
         # delete the events
-        retval = session.query(models.Event.id).filter(
+        retval = session.query(models.Event).filter(
             models.Event.id <= max_id).filter(
                 models.Event.stack_id == stack_id).delete()
 
