@@ -100,7 +100,7 @@ def reset_state_on_error(func):
     return handle_exceptions
 
 
-class Stack(collections.Mapping):
+class Stack(collections.abc.Mapping):
 
     ACTIONS = (
         CREATE, DELETE, UPDATE, ROLLBACK, SUSPEND, RESUME, ADOPT,
@@ -126,7 +126,7 @@ class Stack(collections.Mapping):
                  nested_depth=0, strict_validate=True, convergence=False,
                  current_traversal=None, tags=None, prev_raw_template_id=None,
                  current_deps=None, cache_data=None,
-                 deleted_time=None, converge=False):
+                 deleted_time=None, converge=False, refresh_cred=False):
 
         """Initialise the Stack.
 
@@ -188,6 +188,9 @@ class Stack(collections.Mapping):
         self.thread_group_mgr = None
         self.converge = converge
 
+        # This flag is use to check whether credential needs to refresh or not
+        self.refresh_cred = refresh_cred
+
         # strict_validate can be used to disable value validation
         # in the resource properties schema, this is useful when
         # performing validation when properties reference attributes
@@ -541,10 +544,35 @@ class Stack(collections.Mapping):
                                 {'res': str(res),
                                  'err': str(exc)})
 
+    @classmethod
+    def _check_refresh_cred(cls, context, stack):
+        if stack.user_creds_id:
+            creds_obj = ucreds_object.UserCreds.get_by_id(
+                context, stack.user_creds_id)
+            creds = creds_obj.obj_to_primitive()["versioned_object.data"]
+            stored_context = common_context.StoredContext.from_dict(creds)
+
+            if cfg.CONF.deferred_auth_method == 'trusts':
+                old_trustor_proj_id = stored_context.tenant_id
+                old_trustor_user_id = stored_context.trustor_user_id
+
+                trustor_user_id = context.auth_plugin.get_user_id(
+                    context.clients.client('keystone').session)
+                trustor_proj_id = context.auth_plugin.get_project_id(
+                    context.clients.client('keystone').session)
+                return False if (
+                    old_trustor_user_id == trustor_user_id) and (
+                    old_trustor_proj_id == trustor_proj_id
+                    ) else True
+
+        # Should not raise error or allow refresh credential when we can't find
+        # user_creds_id in stack
+        return False
+
     @classmethod
     def load(cls, context, stack_id=None, stack=None, show_deleted=True,
              use_stored_context=False, force_reload=False, cache_data=None,
-             load_template=True):
+             load_template=True, check_refresh_cred=False):
         """Retrieve a Stack from the database."""
         if stack is None:
             stack = stack_object.Stack.get_by_id(
@@ -555,13 +583,22 @@ class Stack(collections.Mapping):
             message = _('No stack exists with id "%s"') % str(stack_id)
             raise exception.NotFound(message)
 
+        refresh_cred = False
+        if check_refresh_cred and (
+            cfg.CONF.deferred_auth_method == 'trusts'
+        ):
+            if cls._check_refresh_cred(context, stack):
+                use_stored_context = False
+                refresh_cred = True
+
         if force_reload:
             stack.refresh()
 
         return cls._from_db(context, stack,
                             use_stored_context=use_stored_context,
                             cache_data=cache_data,
-                            load_template=load_template)
+                            load_template=load_template,
+                            refresh_cred=refresh_cred)
 
     @classmethod
     def load_all(cls, context, limit=None, marker=None, sort_keys=None,
@@ -595,7 +632,7 @@ class Stack(collections.Mapping):
     @classmethod
     def _from_db(cls, context, stack,
                  use_stored_context=False, cache_data=None,
-                 load_template=True):
+                 load_template=True, refresh_cred=False):
         if load_template:
             template = tmpl.Template.load(
                 context, stack.raw_template_id, stack.raw_template)
@@ -619,7 +656,8 @@ class Stack(collections.Mapping):
                    prev_raw_template_id=stack.prev_raw_template_id,
                    current_deps=stack.current_deps, cache_data=cache_data,
                    nested_depth=stack.nested_depth,
-                   deleted_time=stack.deleted_at)
+                   deleted_time=stack.deleted_at,
+                   refresh_cred=refresh_cred)
 
     def get_kwargs_for_cloning(self, keep_status=False, only_db=False,
                                keep_tags=False):
@@ -687,6 +725,17 @@ class Stack(collections.Mapping):
             s['raw_template_id'] = self.t.id
 
         if self.id is not None:
+            if self.refresh_cred:
+                keystone = self.clients.client('keystone')
+                trust_ctx = keystone.regenerate_trust_context()
+                new_creds = ucreds_object.UserCreds.create(trust_ctx)
+                s['user_creds_id'] = new_creds.id
+
+                self._delete_user_cred(raise_keystone_exception=True)
+
+                self.user_creds_id = new_creds.id
+                self.refresh_cred = False
+
             if exp_trvsl is None and not ignore_traversal_check:
                 exp_trvsl = self.current_traversal
 
@@ -1338,6 +1387,11 @@ class Stack(collections.Mapping):
         Update will fail if it exceeds the specified timeout. The default is
         60 minutes, set in the constructor
         """
+        # Populate resource data needed for calulating frozen definitions
+        # (particularly for metadata, which doesn't get stored separately).
+        self._update_all_resource_data(for_resources=True,
+                                       for_outputs=False)
+
         self.updated_time = oslo_timeutils.utcnow()
         updater = scheduler.TaskRunner(self.update_task, newstack,
                                        msg_queue=msg_queue, notify=notify)
@@ -1495,7 +1549,10 @@ class Stack(collections.Mapping):
                     # Rolling back to previous resource
                     score += 10
 
-                return score, ext_rsrc.updated_at
+                last_changed_at = ext_rsrc.updated_at
+                if last_changed_at is None:
+                    last_changed_at = ext_rsrc.created_at
+                return score, last_changed_at
 
             candidates = sorted((r for r in self.ext_rsrcs_db.values()
                                  if r.name == rsrc_name),
@@ -1840,11 +1897,10 @@ class Stack(collections.Mapping):
             LOG.exception("Failed to retrieve user_creds")
             return None
 
-    def _delete_credentials(self, stack_status, reason, abandon):
+    def _delete_user_cred(self, stack_status=None, reason=None,
+                          raise_keystone_exception=False):
         # Cleanup stored user_creds so they aren't accessible via
         # the soft-deleted stack which remains in the DB
-        # The stack_status and reason passed in are current values, which
-        # may get rewritten and returned from this method
         if self.user_creds_id:
             user_creds = self._try_get_user_creds()
             # If we created a trust, delete it
@@ -1874,6 +1930,8 @@ class Stack(collections.Mapping):
                         # Without this, they would need to issue
                         # an additional stack-delete
                         LOG.exception("Error deleting trust")
+                        if raise_keystone_exception:
+                            raise
 
             # Delete the stored credentials
             try:
@@ -1883,13 +1941,18 @@ class Stack(collections.Mapping):
                 LOG.info("Tried to delete user_creds that do not exist "
                          "(stack=%(stack)s user_creds_id=%(uc)s)",
                          {'stack': self.id, 'uc': self.user_creds_id})
+            self.user_creds_id = None
+        return stack_status, reason
 
-            try:
-                self.user_creds_id = None
-                self.store()
-            except exception.NotFound:
-                LOG.info("Tried to store a stack that does not exist %s",
-                         self.id)
+    def _delete_credentials(self, stack_status, reason, abandon):
+        # The stack_status and reason passed in are current values, which
+        # may get rewritten and returned from this method
+        stack_status, reason = self._delete_user_cred(stack_status, reason)
+        try:
+            self.store()
+        except exception.NotFound:
+            LOG.info("Tried to store a stack that does not exist %s",
+                     self.id)
 
         # If the stack has a domain project, delete it
         if self.stack_user_project_id and not abandon:
@@ -1928,8 +1991,8 @@ class Stack(collections.Mapping):
 
         stack_status = self.COMPLETE
         reason = 'Stack %s completed successfully' % action
-        self.state_set(action, self.IN_PROGRESS, 'Stack %s started' %
-                       action)
+        self.state_set(action, self.IN_PROGRESS, 'Stack %s started at %s' %
+                       (action, oslo_timeutils.utcnow().isoformat()))
         if notify is not None:
             notify.signal()
 
