@@ -13,11 +13,10 @@
 
 import datetime
 import json
+from unittest import mock
 import uuid
 
-import mock
 from oslo_utils import timeutils
-import six
 from swiftclient import client as swiftclient_client
 from swiftclient import exceptions as swiftclient_exceptions
 from testtools import matchers
@@ -25,6 +24,7 @@ from testtools import matchers
 from heat.common import exception
 from heat.common import template_format
 from heat.engine.clients.os import swift
+from heat.engine import node_data
 from heat.engine import resource
 from heat.engine import rsrc_defn
 from heat.engine import scheduler
@@ -113,9 +113,6 @@ def cont_index(obj_name, num_version_hist):
 
 
 class SwiftSignalHandleTest(common.HeatTestCase):
-    def setUp(self):
-        super(SwiftSignalHandleTest, self).setUp()
-        utils.setup_dummy_db()
 
     @mock.patch.object(swift.SwiftClientPlugin, '_create')
     @mock.patch.object(resource.Resource, 'physical_resource_name')
@@ -139,7 +136,7 @@ class SwiftSignalHandleTest(common.HeatTestCase):
         obj_name = "%s-%s-abcdefghijkl" % (st.name, handle.name)
         regexp = ("http://fake-host.com:8080/v1/AUTH_test_tenant/%s/test_st-"
                   "test_wait_condition_handle-abcdefghijkl"
-                  "\?temp_url_sig=[0-9a-f]{40}&temp_url_expires=[0-9]{10}"
+                  r"\?temp_url_sig=[0-9a-f]{40,64}&temp_url_expires=[0-9]{10}"
                   % st.id)
         res_id = st.resources['test_wait_condition_handle'].resource_id
         self.assertEqual(res_id, handle.physical_resource_name())
@@ -203,7 +200,7 @@ class SwiftSignalHandleTest(common.HeatTestCase):
                                 scheduler.TaskRunner(rsrc.delete))
         self.assertEqual('ClientException: '
                          'resources.test_wait_condition_handle: '
-                         'Overlimit: 413', six.text_type(exc))
+                         'Overlimit: 413', str(exc))
 
     @mock.patch.object(swift.SwiftClientPlugin, '_create')
     @mock.patch.object(resource.Resource, 'physical_resource_name')
@@ -234,7 +231,7 @@ class SwiftSignalHandleTest(common.HeatTestCase):
                                 scheduler.TaskRunner(rsrc.delete))
         self.assertEqual('ClientException: '
                          'resources.test_wait_condition_handle: '
-                         'Overlimit: 413', six.text_type(exc))
+                         'Overlimit: 413', str(exc))
 
     @mock.patch.object(swift.SwiftClientPlugin, '_create')
     @mock.patch.object(resource.Resource, 'physical_resource_name')
@@ -283,22 +280,21 @@ class SwiftSignalHandleTest(common.HeatTestCase):
         self.assertEqual(old_url, rsrc.FnGetRefId())
 
     def test_swift_handle_refid_convergence_cache_data(self):
-        cache_data = {'test_wait_condition_handle': {
-            'uuid': mock.ANY,
-            'id': mock.ANY,
-            'action': 'CREATE',
-            'status': 'COMPLETE',
-            'reference_id': 'convg_xyz'
-        }}
+        cache_data = {
+            'test_wait_condition_handle': node_data.NodeData.from_dict({
+                'uuid': mock.ANY,
+                'id': mock.ANY,
+                'action': 'CREATE',
+                'status': 'COMPLETE',
+                'reference_id': 'convg_xyz'
+            })
+        }
         st = create_stack(swiftsignalhandle_template, cache_data=cache_data)
-        rsrc = st['test_wait_condition_handle']
+        rsrc = st.defn['test_wait_condition_handle']
         self.assertEqual('convg_xyz', rsrc.FnGetRefId())
 
 
 class SwiftSignalTest(common.HeatTestCase):
-    def setUp(self):
-        super(SwiftSignalTest, self).setUp()
-        utils.setup_dummy_db()
 
     @mock.patch.object(swift.SwiftClientPlugin, '_create')
     @mock.patch.object(resource.Resource, 'physical_resource_name')
@@ -331,7 +327,7 @@ class SwiftSignalTest(common.HeatTestCase):
 
         st.create()
         self.assertIn('not a valid SwiftSignalHandle.  The Swift TempURL path',
-                      six.text_type(st.status_reason))
+                      str(st.status_reason))
 
     @mock.patch.object(swift.SwiftClientPlugin, 'get_signal_url')
     def test_validate_handle_url_bad_container_name(self, mock_handle_url):
@@ -344,7 +340,7 @@ class SwiftSignalTest(common.HeatTestCase):
 
         st.create()
         self.assertIn('not a valid SwiftSignalHandle.  The container name',
-                      six.text_type(st.status_reason))
+                      str(st.status_reason))
 
     @mock.patch.object(swift.SwiftClientPlugin, '_create')
     @mock.patch.object(resource.Resource, 'physical_resource_name')
@@ -394,7 +390,7 @@ class SwiftSignalTest(common.HeatTestCase):
 
         time_now = timeutils.utcnow()
         time_series = [datetime.timedelta(0, t) + time_now
-                       for t in six.moves.xrange(1, 100)]
+                       for t in range(1, 100)]
         timeutils.set_time_override(time_series)
         self.addCleanup(timeutils.clear_time_override)
 
@@ -721,8 +717,9 @@ class SwiftSignalTest(common.HeatTestCase):
         st.create()
         self.assertEqual(('CREATE', 'COMPLETE'), st.state)
         expected = ('http://fake-host.com:8080/v1/AUTH_test_tenant/%s/'
-                    'test_st-test_wait_condition_handle-abcdefghijkl\?temp_'
-                    'url_sig=[0-9a-f]{40}&temp_url_expires=[0-9]{10}') % st.id
+                    r'test_st-test_wait_condition_handle-abcdefghijkl\?temp_'
+                    'url_sig=[0-9a-f]{40,64}&'
+                    'temp_url_expires=[0-9]{10}') % st.id
         self.assertThat(handle.FnGetAtt('endpoint'),
                         matchers.MatchesRegex(expected))
 
@@ -752,7 +749,7 @@ class SwiftSignalTest(common.HeatTestCase):
         self.assertEqual(('CREATE', 'COMPLETE'), st.state)
         expected = ("curl -i -X PUT 'http://fake-host.com:8080/v1/"
                     "AUTH_test_tenant/%s/test_st-test_wait_condition_"
-                    "handle-abcdefghijkl\?temp_url_sig=[0-9a-f]{40}&"
+                    r"handle-abcdefghijkl\?temp_url_sig=[0-9a-f]{40,64}&"
                     "temp_url_expires=[0-9]{10}'") % st.id
         self.assertThat(handle.FnGetAtt('curl_cli'),
                         matchers.MatchesRegex(expected))
@@ -916,9 +913,10 @@ class SwiftSignalTest(common.HeatTestCase):
         mock_name.return_value = obj_name
         mock_swift_object.get_container.return_value = cont_index(obj_name, 2)
         mock_swift_object.get_object.side_effect = (
-            (obj_header, ''),
             swiftclient_client.ClientException(
-                "Object %s not found" % obj_name, http_status=404)
+                "Object %s not found" % obj_name, http_status=404),
+            (obj_header, '{"id": 1}'),
+            (obj_header, '{"id": 2}'),
         )
 
         st.create()
