@@ -90,6 +90,32 @@ def dummy_context(user='test_username', tenant_id='test_tenant_id',
     })
 
 
+def dummy_system_admin_context():
+    """Return a heat.common.context.RequestContext for system-admin.
+
+    :returns: an instance of heat.common.context.RequestContext
+
+    """
+    ctx = dummy_context(roles=['admin', 'member', 'reader'])
+    ctx.system_scope = 'all'
+    ctx.project_id = None
+    ctx.tenant_id = None
+    return ctx
+
+
+def dummy_system_reader_context():
+    """Return a heat.common.context.RequestContext for system-reader.
+
+    :returns: an instance of heat.common.context.RequestContext
+
+    """
+    ctx = dummy_context(roles=['reader'])
+    ctx.system_scope = 'all'
+    ctx.project_id = None
+    ctx.tenant_id = None
+    return ctx
+
+
 def parse_stack(t, params=None, files=None, stack_name=None,
                 stack_id=None, timeout_mins=None,
                 cache_data=None, tags=None):
@@ -194,15 +220,19 @@ class JsonRepr(object):
 
 
 class ForeignKeyConstraintFixture(fixtures.Fixture):
-    def __init__(self, sqlite_fk=True):
-        self.enable_fkc = sqlite_fk
+
+    def __init__(self):
+        self.engine = get_engine()
 
     def _setUp(self):
-        new_context = db_api.db_context.make_new_manager()
-        new_context.configure(sqlite_fk=self.enable_fkc)
+        if self.engine.name == 'sqlite':
+            self.engine.execute("PRAGMA foreign_keys=ON")
 
-        self.useFixture(fixtures.MockPatchObject(db_api, '_facade', None))
-        self.addCleanup(db_api.db_context.patch_factory(new_context._factory))
+            def disable_fks():
+                with self.engine.connect() as conn:
+                    conn.connection.rollback()
+                    conn.execute("PRAGMA foreign_keys=OFF")
+            self.addCleanup(disable_fks)
 
 
 class AnyInstance(object):
